package cn.sh.stc.sict.theme.hpgp.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.sh.stc.sict.theme.hpgp.model.HpgpBusyIdlePrediction;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hpgp.service.HpgpBusyIdlePredictionService;
import cn.sh.stc.sict.theme.hpgp.service.HpgpDepartmentRankService;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.model.HphyDoctor;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.sh.stc.sict.theme.hpgp.dao.HpHzjlMapper;
import cn.sh.stc.sict.theme.hpgp.model.HpHzjl;
import cn.sh.stc.sict.theme.hpgp.service.HpHzjlService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * (HpHzjl)表服务实现类
 *
 * @author F_xh
 * @since 2022-08-15 14:14:33
 */
@AllArgsConstructor
@Service("hpHzjlService")
public class HpHzjlServiceImpl extends ServiceImpl<HpHzjlMapper, HpHzjl> implements HpHzjlService {

    private final HpgpDepartmentRankService hpgpDepartmentRankService;
    private final HpgpBusyIdlePredictionService hpgpBusyIdlePredictionService;
    private final HpDocInfoService hpDocInfoService;

    /**
     * 根据身份证号查找最后三次不同科室就诊记录，并计算忙闲
     * 如果未找到，则推送三个热门科室
     *
     * @param certId
     * @return
     */
    @Override
    public List<HpgpBusyIdlePrediction> listByCertId(String certId) {
        List<HpgpBusyIdlePrediction> result = new ArrayList<>();
        DateTime startTime = DateUtil.tomorrow();
        DateTime endTime = DateUtil.offsetDay(startTime, 6);

        LambdaQueryWrapper<HpHzjl> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(HpHzjl::getZjhm, certId)
                .groupBy(HpHzjl::getJzksbm)
                .orderByDesc(HpHzjl::getJzksrq)
                .last("limit 10");
        List<HpHzjl> recordList = this.list(wrapper);
        if (CollUtil.isNotEmpty(recordList)) {
            for (HpHzjl record : recordList) {
                if (result.size() >= 20) {
                    break;
                }
                // 查找科室
                HpgpDepartmentRank dept = hpgpDepartmentRankService.getByCode(record.getYljgdm(), null, record.getJzksbm());
                if (ObjectUtil.isEmpty(dept)) {
                    continue;
                }
                // 忙闲预测
                result.addAll(hpgpBusyIdlePredictionService.listByDeptRank(dept, startTime, endTime));
            }
        }
        if (CollUtil.isEmpty(result)) {
            List<HpgpDepartmentRank> rankList = hpgpDepartmentRankService.listHotDept();
            rankList.forEach(rank -> {
                result.addAll(hpgpBusyIdlePredictionService.listByDeptRank(rank, startTime, endTime));
            });
        }

        return result;
    }

    @Override
    public List<HpDocInfo> listDoctorByCertId(String certId) {
        LambdaQueryWrapper<HpHzjl> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(HpHzjl::getZjhm, certId)
                .groupBy(HpHzjl::getYljgdm, HpHzjl::getJzksbm, HpHzjl::getZzysgh)
                .orderByDesc(HpHzjl::getJzksrq)
                .last("limit 10");
        List<HpHzjl> recordList = this.list(wrapper);
        List<HpDocInfo> doctorList = new ArrayList<>();
        if (CollUtil.isNotEmpty(recordList)) {
            recordList.forEach(r -> {
                LambdaQueryWrapper<HpDocInfo> dw = Wrappers.lambdaQuery();
                dw.eq(HpDocInfo::getHosOrgCode, r.getYljgdm())
                        .eq(HpDocInfo::getResourceCode, r.getZzysgh())
                        .eq(HpDocInfo::getResourceName, r.getZzysxm())
                        .last("limit 1");
                HpDocInfo one = hpDocInfoService.getOne(dw);
                doctorList.add(one);
            });
        }
        return doctorList;
    }
}
